var encoding = require("./encoding");
var RSAKey = require("./rsa_sign").RSAKey;

var url=require('url');
var querystring = require("querystring");
var sys=require('sys');


var api = {
	baseUrl: 'http://dev2.glbse.com:3000/',
	useProxy: false,
	debug: true
};

api.getNonce = function () {
	return Math.floor(Math.random() * 1000000000000);
}

api.encodeRequest = function (data) {
	var jtext = JSON.stringify(data);
	var data64 = encoding.encodeBase64(jtext, -1);

	return data64;
}

api.sign_and_encode = function (key, data) {
	var hSig = key.signString(data, 'sha512');
	var bSig = encoding.decodeHex(hSig);
	var b64Sig = encoding.encodeBase64(bSig);

	return b64Sig;
}

api.decodeResponse = function (d) {
	var json = encoding.decodeBase64(d);
	var resp;
	try {
		resp = JSON.parse(json);
	}
	catch (err) {
		if (this.debug && (typeof d === 'string') && (d.indexOf('DOCTYPE') > 0)) {
			var win = window.open('', 'error');
			win.document.write(d);
		}
		resp = {
			errmsg: 'There was a server side error.'
		};
	}
	return resp;
}

api.getApiKey = function (acct) {
	var keys = {
		id: acct.id,
		rsaKey: new RSAKey()
	};
	keys.rsaKey.readPrivateKeyFromPEMString(acct.privateKey);
	return keys;
}

api.executeSimpleApiCall = function (method, apiKey, callbacks) {
	var data = {
		'userid': apiKey.id,
		'nonce': this.getNonce()
	};

	api.executeApiCall(method, apiKey.rsaKey, data, callbacks);
}

api.baseExecuteApiCall = function (parsedUrl, postData, callbacks) {
	var port = parsedUrl.port;
	var client;
	if(parsedUrl.protocol == "https:") {
		port = port ? port : 443;
		client = require('https');
	} else {
		port = port ? port : 80;
		client = require('http');
	}
	var options = {
			  host: parsedUrl.hostname,
			  port: port,
			  path: parsedUrl.pathname,
			  method: 'POST',
			  headers: {
	          	"Host": parsedUrl.hostname,
				"Content-Type": "application/x-www-form-urlencoded",
		     	"Content-Length": postData.length
	      	  }
	    };
		
	var request = client.request(options, function(response) {
		var buffer = "";
		response.on('data', function (chunk) {		    
			buffer += chunk;
		  });
		response.on('end', function () {
			var data = api.decodeResponse(buffer);
			if (data.hasOwnProperty('errmsg')) {
				callbacks.error(data.errmsg);
			} else {
				callbacks.success(data);
			}
		});
		if(response.statusCode != 200)
			callbacks.failure();
	});
	
	request.on('error', function(res) {		
		callbacks.error(res);	
	});
	request.end(postData);
}

api.executeApiCall = function (method, key, data, callbacks) {
	var destUrl = api.baseUrl + method;
	if (api.useProxy) {
		var safeurl = escape(destUrl);
		destUrl = 'proxy.php?url=' + safeurl;
	}
	var d = api.encodeRequest(data);
	var s = api.sign_and_encode(key, d);

	var postData = querystring.stringify({ data: d, signature: s});
	var parsedUrl = url.parse(destUrl);
	api.baseExecuteApiCall(parsedUrl, postData, callbacks);
}

api.generateKeys = function (callbacks) {
	var destUrl = api.baseUrl + 'generate_keypairs';
	if (api.useProxy) {
		var safeurl = escape(destUrl);
		destUrl = 'proxy.php?url=' + safeurl;
	}

	var parsedUrl = url.parse(destUrl);
	api.baseExecuteApiCall(parsedUrl, "", callbacks);
}

api.registerAccount = function (publicKey, callbacks) {
	var destUrl = api.baseUrl + 'register';
	if (this.useProxy) {
		var safeurl = escape(destUrl);
		destUrl = 'proxy.php?url=' + safeurl;
	}

	var data = {
		'public_key': publicKey
	};

	var postData = querystring.stringify({ data: api.encodeRequest(data)});
	var parsedUrl = url.parse(destUrl);
	api.baseExecuteApiCall(parsedUrl, postData, callbacks);
}

api.getBalance = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'account/balance',
		apiKey,
		callbacks
	);
}

api.getDepositAddress = function(apiKey, callbacks) {
	api.executeSimpleApiCall(
		'account/deposit',
		apiKey,
		callbacks
	);
}

api.withdrawFunds = function (address, amount, apiKey, callbacks) {
	var url = 'account/withdraw';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'amount': amount,
		'address': address
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.getPortfolio = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'account/asset/list',
		apiKey,
		callbacks
	);
}

api.getOrders = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'market/orders',
		apiKey,
		callbacks
	);
}

api.cancelOrder = function (orderId, apiKey, callbacks) {
	var url = "market/order/cancel";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"order_id": orderId
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.getBitcoinHistory = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'history/bitcoin',
		apiKey,
		callbacks
	);
}

api.getAssetsHistory = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'history/assets',
		apiKey,
		callbacks
	);
}

api.getMarketHistory = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'history/market',
		apiKey,
		callbacks
	);
}

api.getMotions = function (apiKey, callbacks) {
	api.executeSimpleApiCall(
		'motions',
		apiKey,
		callbacks
	);
}

api.getAssetDetails = function (assetId, apiKey, callbacks) {
	var url = 'asset';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'asset_id': assetId
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.getAssetDepth = function (assetId, apiKey, callbacks) {
	var url = 'asset/depth';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'asset_id': assetId
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.getMotionDetails = function (motionId, apiKey, callbacks) {
	var url = 'asset/motion';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'motion_id': motionId
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.getMotionTally = function (motionId, apiKey, callbacks) {
	var url = "vote/tally";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"motion_id": motionId
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.doBuy = function (asset, quantity, price, apiKey, callbacks) {
	var url = 'market/buy';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'asset': asset,
		'quantity': quantity,
		'price': price
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.doSell = function (asset, quantity, price, apiKey, callbacks) {
	var url = 'market/sell';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'asset': asset,
		'quantity': quantity,
		'price': price
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.doTransfer = function (asset, quantity, toAccount, apiKey, callbacks) {
	var url = 'account/asset/transfer';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		'asset': asset,
		'quantity': quantity,
		'to_account': toAccount
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.doMotionVote = function (motionId, vote, apiKey, callbacks) {
	var url = 'vote';
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"motion_id": motionId,
		"vote": vote
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.doIssueAsset = function (assetId, quantity, apiKey, callbacks) {
	var url = "account/asset/issue";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"asset_id": assetId,
		"quantity": quantity
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.registerTicker = function (assetId, tickerSymbol, apiKey, callbacks) {
	var url = "register/ticker";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"asset_id": assetId,
		"ticker": tickerSymbol
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.createNewAsset = function (contract, signature, apiKey, callbacks) {
	var url = "account/asset/new";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"contract": contract,
		"contract_signature": signature
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.submitNewMotion = function (assetId, text, expiration, apiKey, callbacks) {
	var url = "vote/new";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"asset_id": assetId,
		"motion": text,
		"date": expiration
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

api.doPayAssetHolders = function (assetId, amount, apiKey, callbacks) {
	var url = "asset/pay";
	var data = {
		'userid': apiKey.id,
		'nonce': api.getNonce(),
		"asset_id": assetId,
		"amount": amount
	};
	api.executeApiCall(url, apiKey.rsaKey, data, callbacks);
}

exports.Api = api;